#ifndef __MATTE__
#define __MATTE__

#include "Material.h"

#include "Lambertian.h"
#include "Ambient.h"

class Matte{
		
	public:
		Matte(void);
		
		void set_ka(const float k);
		
		void set_kd(const float k);
		
		void set_cd(const RGBColor &c);
		
		virtual RGBColor shade (ShadeRec &sr);
	
	private:
		Lambertian * ambient_brdf;
		Lambertian * diffuse_brdf;
	
};

Matte::Matte(void){
 	ambient_brdf = new Lambertian;
	diffuse_brdf = new Lambertian;
}
	
void Matte :: set_ka(const float ka){
	ambient_brdf->set_kd(ka);		
}

void Matte :: set_kd(const float kd){
	diffuse_brdf->set_kd(kd);		
}

void Matte :: set_cd(const RGBColor &c){
	ambient_brdf->set_cd(c);
	diffuse_brdf->set_cd(c);
}

RGBColor Matte::shade(ShadeRec &sr){
	int NumLights = sr.w.lights.size();
	Vector3D wo	=  -sr.ray.d;
	RGBColor my_rho = ambient_brdf->rho(sr,wo);
	RGBColor my_light_color = sr.w.ambient_ptr->L(sr);
	
	//sr.w.ambient_ptr->Test_Ambient();
	RGBColor myColor =  ambient_brdf->rho(sr,wo) * sr.w.ambient_ptr->L(sr);
	
	/* Looks OK.
	cout << "sr.w.ambient_ptr->L(sr).r = " << sr.w.ambient_ptr->L(sr).r << "\n";
	cout << "sr.w.ambient_ptr->L(sr).g = " << sr.w.ambient_ptr->L(sr).g << "\n";
	cout << "sr.w.ambient_ptr->L(sr).b = " << sr.w.ambient_ptr->L(sr).b << "\n";
	*/
	
	for(int j = 0; j < NumLights; j++){
		Vector3D wi = sr.w.lights[j]->get_direction(sr);
		
		float ndotwi = sr.normal * wi;
		// I wasn't setting the normal in ShaderRec!
		
		///TODO: I am having trouble getting the light RGB color from my Vector<LIGHT> in here.
		
		if (ndotwi <= 0.0)
		{
			myColor = black;
		}
		
		if (ndotwi > 0.0){
			//cout << "Light hit object";
			/// TODO: This is bad. It is hardcoded for a (1.0,1.0,1.0) light
			//myColor += diffuse_brdf->f(sr,wo,wi) * sr.w.lights[j]->L(sr) * ndotwi;
						
			myColor = 2 * diffuse_brdf->f(sr,wo,wi) * ndotwi;
		}		
	}
	
	return (myColor);
	
}

#endif

