#ifndef __PHONG__
#define __PHONG__

class Phong{
	public:
		Phong(void);
			
		virtual RGBColor shade(ShadeRec &s);
		void set_ka(const float input_ka);
		void set_kd(const float input_kd);
		void set_exp(const float input_exp);
		void set_cd(const RGBColor &c);
	
	protected:
		Lambertian *ambient_brdf;
		Lambertian *diffuse_brdf;
};

Phong :: Phong(void){
	ambient_brdf = new Lambertian;
	diffuse_brdf = new Lambertian;
	
}

void Phong :: set_ka(const float input_ka){
	ambient_brdf->set_kd(input_ka);
}

void Phong :: set_kd(const float kd){
	diffuse_brdf->set_kd(kd);		
}

void Phong :: set_cd(const RGBColor &c){
	ambient_brdf->set_cd(c);
	diffuse_brdf->set_cd(c);
}

void Phong :: set_exp(const float input_exp){
	//specular_brdf->set_exp(input_exp);	
}

RGBColor Phong :: shade(ShadeRec &sr){
	Vector3D wo= -sr.ray.d;
	RGBColor my_color = ambient_brdf->rho(sr,wo) * sr.w.ambient_ptr->L(sr);
	
	int num_lights = sr.w.lights.size();
	
	for(int j = 0; j < num_lights; j++){
		Vector3D light_to_hitPoint = sr.w.lights[j]->get_direction(sr);
		
		float normal_dot_light = sr.normal * light_to_hitPoint;
		
		if (normal_dot_light <= 0.0)
		{
			my_color = black;
		}
		
		if (normal_dot_light > 0.0){
			// OK, here's how to actually do Phong Shading:
			/*
			kd (L * N) id       + (ks(R * V) ^ alpha) * ls 
			diffifuse component + (specular reflection constant * (reflected ray of light * direction towards viewer)
				^ specular power * specular light component	

			*/

			// This looks good when I have my matte shader running
			RGBColor DiffuseComponenet = (diffuse_brdf->f(sr,wo,light_to_hitPoint) * normal_dot_light);

			// The vector describing the reflected light is simply
			// the negative of the vector describing the light
			// hitting the surface
			Vector3D light_reflected = - light_to_hitPoint;
			light_reflected.normalize();
	
			//wo = point to eye
			wo.normalize();
		
			///TODO: Set this in the class and make it variable
			float specular_power = 3.0;


	
			double specular_term;
	
			double tmp = wo * light_reflected;

			if (tmp >= .5){
//				cout << "wo * light_reflected = " << tmp << "\n";
			}
			 specular_term = pow((double)(wo * light_reflected), (double)specular_power);

			//cout << "normal_dot_light = " << normal_dot_light << "\n";

			//specular_term = specular_term * 10000000;
	
			//cout << "specular_term = " << specular_term << "\n";



			RGBColor SpecularColor = RGBColor(1.0, 1.0, 1.0) * specular_term;

			my_color = DiffuseComponenet + SpecularColor;
		}
		
	}
	
	
	
	
	return my_color;
}

#endif
