#include "Sphere.cpp"
#include "Plane.cpp"
#include "Triangle.cpp"
#include <fstream>
#include "World.h"
#include "myViewPlane.cpp"
#include "myViewPlane.h"
#include "from_book/Point2D.cpp"
#include "Ambient.h"
#include "Light.h"
#include "PointLight.h"
//#include "Material.h"
#include "Matte.h"
#include "GlossySpecular.h"
#include "Phong.h"


ShadeRec World :: hit_bare_bones_objects(const Ray & ray) {
	ShadeRec sr (*this);
	double t;
	Normal normal;
	Point3D local_hit_point;
	double tmin	= kHugeValue;
	int num_objects = objects.size();
	
	sr.hit_an_object = false;
	
	for (int j = 0; j < num_objects; j++){		
		if (objects[j]->hit(ray, t, &sr) && (t < tmin)){

			sr.hit_an_object = true;
			tmin = t;
			sr.color = objects[j]->get_color(); // Just for the older method
			sr.material_ptr = objects[j]->material;
			sr.hit_point = ray.o + t * ray.d;
			normal = sr.normal;
			
			/*
			cout << "sr.normal.x = " << sr.normal.x << "\n";
			cout << "sr.normal.y = " << sr.normal.y << "\n";
			cout << "sr.normal.z = " << sr.normal.z << "\n\n";
			*/
			
			local_hit_point = sr.local_hit_point;
			
		}
	}
	if (sr.hit_an_object){
		sr.t = tmin;
		sr.normal = normal;
		sr.local_hit_point = local_hit_point;
	}
	return (sr);
}


int main(){
	World myWorld;
	
	myWorld.build();
	//myWorld.build_example();
	myWorld.charCounter = 0;
		
	Ray ray;
	for (int r = 0; r < myWorld.ViewPlane.vres; r++) // up
	{
		// across
		for (int c = 0 ; c< myWorld.ViewPlane.hres; c++){
			
			ray.o = myWorld.ViewPlane.eyePoint;
			ray.d = myWorld.ViewPlane.from_screen_to_world(c, r) - myWorld.ViewPlane.eyePoint;
			ray.d.normalize();
			
			ShadeRec myShadeRec = myWorld.hit_bare_bones_objects(ray);
			// I need to use ray_trace p. 264 here.
			
			RGBColor tmpColor = myWorld.background_color;
	
			if (myShadeRec.hit_an_object == true){

				Phong * myMaterial = myShadeRec.material_ptr; 
				
				tmpColor = myMaterial->shade(myShadeRec);
			}
		

			if (tmpColor.r > 1){
				tmpColor.r = 1.0;
			}
			if (tmpColor.g > 1){
				tmpColor.g = 1.0;
			}
			if (tmpColor.b > 1){
				tmpColor.b = 1.0;
			}

			/*
			cout << "tmpColor.r = " << tmpColor.r << "\n";
			cout << "tmpColor.g = " << tmpColor.g << "\n";
			cout << "tmpColor.b = " << tmpColor.b << "\n";
			*/		

			myWorld.myfile << " " << (ceil( 255 * tmpColor.r)) << " " << (ceil( 255 * tmpColor.g)) << " " << (ceil( 255 * tmpColor.b)) << " ";
			
			//cout << " " << (ceil( 255 * tmpColor.r)) << " " << (ceil( 255 * tmpColor.g)) << " " << (ceil( 255 * tmpColor.b)) << " ";
			
			myWorld.charCounter = myWorld.charCounter +6; 
			if (myWorld.charCounter >= 40){
				//cout << "NEWLINE PRINTED";
				myWorld.myfile << "\n";
				myWorld.charCounter = 0;
			}
			
			
			
			// printf("%d",mySphere->hit(ray, t, &p));
			//printf("%f ",t); 			
		}
		
		
	} printf("\n");
	myWorld.myfile.close();
}

void World :: build_ppm_file(myViewPlane ViewPlane){
	
	//ofstream myfile;
	myfile.open ("example.ppm");
	myfile << "P3\n";
	myfile << "# Ray tracer output file\n";
	myfile << ViewPlane.hres << " " << ViewPlane.vres << "\n"; // width height
	myfile << "255 \n"; // Max intensity
	

}

void World :: build(void){
	
	
	/********************
	Lighting Stuff
	********************/
	PointLight * light_ptr2 = new PointLight;
	light_ptr2->set_location(Vector3D(4.0,4.0,4.0));
	light_ptr2->scale_radiance(1.0);
	light_ptr2->set_color(RGBColor(1.0, 1.0, 1.0));
	add_light(light_ptr2);
	
	Ambient * ambient_ptr = new Ambient;
	ambient_ptr->scale_radiance(1.0); // default
	set_ambient_light(ambient_ptr);
	
	
	// ViewPlane setup
	ViewPlane.hres =512;
	ViewPlane.vres =512;
	
	build_ppm_file(ViewPlane);
	
	Phong * matte_ptr1 = new Phong;
	matte_ptr1->set_ka(1.00);
	matte_ptr1->set_kd(0.45);
	matte_ptr1->set_cd(RGBColor(0.05, 1.0, 0.05));
	matte_ptr1->set_exp(30.0);
	
	// Sphere at [2,2,0.2] radius 1, diffuse color [.05, 1, .05], specular color [.9, 1, .9], specular power 30
	Point3D sphereCenter = -(2.0, 2.0, 0.2);	
	Sphere *mySphere = new Sphere(sphereCenter, 1.0);
	mySphere->set_color(RGBColor(0.05, 1.0, 0.05));	
	mySphere->set_material(matte_ptr1);
	add_object(mySphere);
	
	
	Phong * matte_ptr2 = new Phong;
	matte_ptr2->set_ka(1.00);
	matte_ptr2->set_kd(1.00);
	matte_ptr2->set_cd(RGBColor(1.0, 0.05, 0.05));  // Red Sphere
	matte_ptr2->set_exp(10.0);
	
	// Sphere at [0,0,1] radius 1, diffuse color [1, .05, .05], specular color [1, .9, .9], specular power 10
	Point3D sphereCenter_2 = (0.0, 0.0, 1.0);	
	Sphere *mySphere_2 = new Sphere(sphereCenter_2, 1.0);
	mySphere_2->set_color(RGBColor(1.0, 0.05, 0.05)); 
	mySphere_2->set_material(matte_ptr2);
	add_object(mySphere_2);
	//red Sphere centered at 0,0,0 with 85 radius.
	
	
	
	// [-1, -1, .02][2,-2,.02][-1.5,-1.5,1.5]
	Phong * matte_ptr3 = new Phong;
	matte_ptr3->set_ka(1.00);
	matte_ptr3->set_kd(1.00);
	matte_ptr3->set_cd(RGBColor(1.0, 1.00, 0.05));
	matte_ptr3->set_exp(100.0);
	
	Triangle *myTriangle = new Triangle (Point3D(-1.0, -1.0, 0.02), Point3D(2.0, -2.0, 0.02), Point3D(-1.5, -1.5, 1.5)); 
	myTriangle->set_color(RGBColor(1.0, 1.0, .05));
	myTriangle->set_material(matte_ptr1);
	add_object(myTriangle);
	
	
	Phong * matte_ptr4 = new Phong;
	matte_ptr4->set_ka(1.00);
	matte_ptr4->set_kd(1.00);
	matte_ptr4->set_cd(RGBColor(0.5, 0.5, 1.00));
	matte_ptr4->set_exp(3.0);	
	
	Plane *myPlane = new Plane(Point3D(0.0, 0.0, 0.0), Normal(0.0, 0.0, 1.0));
	myPlane->set_color(RGBColor(.05, .05, 1.0));
	myPlane->set_material(matte_ptr4);
	add_object(myPlane);
		
	ViewPlane.eyePoint = 1 * (Point3D(0.0, 5.0, 2.0));
	// Place our eye
	
	ViewPlane.eye_at = Point3D(0.0, 0.0, 0.0);
	// set what we are looking at 
	
	ViewPlane.eye_up = Vector3D(0.0, 0.0, 1.0);
	// set our eye_up vector
	
	ViewPlane.distance_to_eye = 1.0;
	ViewPlane.s = 1.0;
	
	

	
	///TODO: This will mess me up later... I am not subclassing material because it was screwing me up.
	
	/*
	Matte * matte_ptr1 = new Matte;
	
	matte_ptr1->set_ka(0.25);
	matte_ptr1->set_kd(0.65);
	matte_ptr1->set_cd(RGBColor(1.0, 0.05, 0.05));
	//Sphere *sphere_ptr1 = new Sphere(Point3D(10,-5,0),27);
	mySphere_2->set_material(matte_ptr1);
	//add_object(sphere_ptr1);
	/********************
	
	********************/
	
}

void World :: set_ambient_light(Ambient *in){
	ambient_ptr = in;
}
